;--------------------------------------------------------------------
;                         MAKE THE R-GRID
;--------------------------------------------------------------------
function make_rgrid,rin,rout,nr,rrefine=rrefine
if not keyword_set(rrefine) then begin
    r = rin*(rout/rin)^(findgen(nr)/(nr-1.d0))
endif else begin
    nextra = rrefine.nspanr*rrefine.nlevr*(2^rrefine.nstepr-1)
    if nextra gt nr-5 then begin
       print,"Sorry not enough nr for this refinement level"
       stop
    endif
    r = rin*(rout/rin)^(findgen(nr-nextra)/(nr-1.d0-nextra))
    for ilev=1,rrefine.nlevr do begin
        for ispan=rrefine.nspanr,1,-1 do begin
            rins=fltarr(2^rrefine.nstepr-1)
            fr=(r(ispan)/r(ispan-1))^(0.5^rrefine.nstepr)
            for i=1,2^rrefine.nstepr-1 do rins(i-1)=r(ispan-1)*fr^i
            r=[r(0:ispan-1),rins,r(ispan:*)]
        endfor
    endfor
 endelse
return,r
end


;--------------------------------------------------------------------
;                       MAKE THE THETA-GRID
;--------------------------------------------------------------------
function make_tgrid,hrgrid,nt,hrgmax=hrgmax,ntex=ntex,$
                    hrlg=hrlg,zrefine=zrefine
if not keyword_set(hrlg) then begin
    thmax = !pi/2.d0
    thmin = !pi/2.d0 - hrgrid
    theta = (thmax-thmin)*findgen(nt)/(nt-1.d0+0.5d0)+thmin
    nnt   = nt
    if keyword_set(hrgmax) then begin
        if not keyword_set(ntex) then begin
            print,'PROBLEM: If you define hrgmax, must also define nextra'
            stop
        endif 
        if hrgmax le hrgrid then begin
            print,'PROBLEM: hrgmax must be larger than hrgrid'
            stop
        endif
        thmax = !pi/2.d0 - hrgrid
        thmin = !pi/2.d0 - hrgmax
        thex=(thmax-thmin)*findgen(ntex)/(ntex*1.d0)+thmin
        theta = [thex,theta]
        nnt   = nt+ntex
    endif
endif else begin
    B     = alog(hrgrid)
    A     = (B-alog(hrlg))/(nt-1.d0)
    theta = !pi/2.d0-exp(-A*findgen(nt)+B)
    nnt   = nt
endelse
if keyword_set(zrefine) then begin
    hunt,theta[*],!pi/2-zrefine.zrref,iz
    theta = [theta[0:iz-1],!pi/2-rotate(dindgen(zrefine.nzref)+0.5,2)*$
             (!pi/2-theta[iz-1])/(1.d0*zrefine.nzref+1.)]
endif
return,theta
end

